package com.jiuqi.syntax;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.syntax.CommonData;
import com.jiuqi.syntax.CommonDatas;
import com.jiuqi.syntax.CustomFunctionListener;
import com.jiuqi.syntax.Function;
import com.jiuqi.syntax.FunctionParam;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.Map;

public final class AutoFunctionListener extends CustomFunctionListener {
  private Object provider;
  
  private String classify;
  
  private Map funcMap;
  
  private static final int SY_FUNC_CALL_ERROR = 10400;
  
  private static final int SY_FUNC_RET_ERROR = 10401;
  
  private static final int SY_FUNC_ACL_ERROR = 10402;
  
  public AutoFunctionListener(Object provider) {
    this(provider, null);
  }
  
  public AutoFunctionListener(Object provider, String classify) {
    this.provider = provider;
    this.classify = classify;
    initFunctions();
  }
  
  private void initFunctions() {
    this.funcMap = new Hashtable<Object, Object>();
    if (this.provider == null)
      return; 
    Class<?> pc = this.provider.getClass();
    Method[] methods = pc.getDeclaredMethods();
    for (int i = 0; i < methods.length; i++) {
      if (Modifier.isPublic(methods[i].getModifiers())) {
        Function func = new Function();
        func.name = methods[i].getName();
        func.resultType = convertType(methods[i].getReturnType());
        if (func.resultType != 0) {
          func.classify = this.classify;
          Class[] paramTypes = methods[i].getParameterTypes();
          int j = 0;
          while (true) {
            if (j >= paramTypes.length) {
              this.functions.add(func);
              this.funcMap.put(func, methods[i]);
              break;
            } 
            FunctionParam param = new FunctionParam();
            param.name = "p" + Integer.toString(j);
            param.type = convertType(paramTypes[j]);
            if (param.type == 0)
              break; 
            param.canOmit = false;
            param.isSign = false;
            func.params.add(param);
            j++;
          } 
        } 
      } 
    } 
  }
  
  private int convertType(Class typeClass) {
    if (int.class.equals(typeClass) || Integer.class.equals(typeClass))
      return 1; 
    if (double.class.equals(typeClass) || Double.class.equals(typeClass))
      return 2; 
    if (String.class.equals(typeClass))
      return 3; 
    if (boolean.class.equals(typeClass) || Boolean.class.equals(typeClass))
      return 5; 
    if (Calendar.class.equals(typeClass))
      return 6; 
    return 0;
  }
  
  public String getClassify() {
    return this.classify;
  }
  
  public int callFunction(String funcName, CommonDatas paramValues, CommonData resultData, boolean isCheckMode) {
    Function func = this.functions.getFunc(funcName);
    if (func == null)
      return 201; 
    Method method = (Method)this.funcMap.get(func);
    try {
      Object[] args = convertParams(paramValues, func);
      Object ret = method.invoke(this.provider, args);
      resultData.type = func.resultType;
      if (!convertObject(ret, resultData))
        return 10401; 
    } catch (com.jiuqi.syntax.AutoFunctionListener.SyntaxException e) {
      return e.getErrorCode();
    } catch (IllegalArgumentException illegalArgumentException) {
      return 202;
    } catch (IllegalAccessException illegalAccessException) {
      return 10402;
    } catch (InvocationTargetException e) {
      e.printStackTrace();
      return 10400;
    } 
    return 0;
  }
  
  private Object[] convertParams(CommonDatas paramValues, Function func) throws SyntaxException {
    Object[] params = new Object[func.params.size()];
    for (int i = 0; i < func.params.size(); i++) {
      CommonData value = (paramValues.size() <= i) ? null : paramValues.getItem(i);
      if (value == null || value.isNull) {
        params[i] = null;
      } else {
        switch ((func.params.getParam(i)).type) {
          case 1:
            params[i] = new Integer((int)value.getIntRealValue());
            break;
          case 2:
            params[i] = new Double(value.getIntRealValue());
            break;
          case 3:
            params[i] = value.cdString;
            break;
          case 5:
            params[i] = new Boolean(value.cdBool);
            break;
          case 6:
            params[i] = value.cdDate;
            break;
          default:
            throw new SyntaxException(this, 202);
        } 
      } 
    } 
    return params;
  }
  
  private boolean convertObject(Object value, CommonData result) {
    if (value == null) {
      result.isNull = true;
      result.initNullValue();
    } else {
      result.isNull = false;
      switch (result.type) {
        case 1:
          if (value instanceof Integer) {
            result.cdInt = ((Integer)value).intValue();
          } else if (value instanceof Double) {
            result.cdInt = ((Double)value).intValue();
          } else {
            return false;
          } 
          return true;
        case 2:
          if (value instanceof Integer) {
            result.cdReal = ((Integer)value).intValue();
          } else if (value instanceof Double) {
            result.cdReal = ((Double)value).doubleValue();
          } else {
            return false;
          } 
        case 3:
          result.cdString = value.toString();
          return true;
        case 5:
          if (value instanceof Boolean) {
            result.cdBool = ((Boolean)value).booleanValue();
          } else {
            return false;
          } 
          return true;
        case 6:
          if (value instanceof Calendar) {
            result.cdDate = (Calendar)value;
          } else if (value instanceof Date) {
            result.setDateValue((Date)value);
          } else if (value instanceof Long) {
            Calendar date = Calendar.getInstance();
            date.setTimeInMillis(((Long)value).longValue());
            result.cdDate = date;
          } else {
            return false;
          } 
          return true;
      } 
      return false;
    } 
    return true;
  }
  
  public String getCodeMessage(int retCode) {
    switch (retCode) {
      case 10400:
        return "函数调用时出现程序异常";
      case 10401:
        return "错误的函数返回值类型";
      case 10402:
        return "类或函数无法访问";
    } 
    return null;
  }
}
